using UnityEngine;
using System.Collections;

public class Trap : MonoBehaviour {
    //
    SpriteManager spriteManager;
    Sprite sprite;

    public int spriteRow = 0;
    public int spriteOff = 0;
    public int spriteOffFrames = 1;  
    public int spriteOn = 1;
    public int spriteOnFrames = 3;
    public bool spriteLoop = true;
    //
    public int spriteFire = 2;
    public int spriteFireFrames = 2;
    //
    public int spriteSize = 32;
    public int spritePixels = 48;
    //
    public bool switchedOn = false;
    //
    public GameObject switchedOnTrigger;
    //
    public bool raiseSelf = false;
    public float raiseAmount = 0;
    float startY = 0;
    float startX = 0;
    float fireY = 0;
    //
    public  Vector3 fireDirection = Vector3.down;
    public Vector3 firePoint = Vector3.down;
    //
    Character target;
    bool disabled = false;
     LayerMask groundLayer, playerLayer;
    //
     public bool canFire = false;
     float fireTime = 0;
     public float frameTime = 0.1f;
    
     float counter = 0;
     int frame = 0;
    //
     public bool isButton = false;
     public bool buttonSwitchOn = false;
     public bool isTrigger = false;
     public bool delaySwitch = false;
    //
     public bool disableIfNoEnemies = false;
     public bool enableIfNoEnemies = false;
    //
    void Awake() {        
        groundLayer = 1 << 9;
        playerLayer = 1 << 10;
    }
	void Start () {
        if (!isTrigger) {
            GameObject uD = GameObject.Find("SpriteManagerProps");
            spriteManager = uD.GetComponent(typeof(SpriteManager)) as SpriteManager;
            sprite = spriteManager.AddSprite(gameObject, spritePixels, spritePixels, spriteOff * spritePixels, (spriteRow + 1) * spritePixels, spritePixels, spritePixels, false);
            transform.eulerAngles = Vector3.zero;
            sprite.SetSizeXY(spriteSize, spriteSize);
            if (!isButton) {
                if (switchedOn) {
                    if (switchedOnTrigger != null) {
                        switchedOnTrigger.gameObject.active = true;
                    }
                } else {
                    if (switchedOnTrigger != null) {
                        switchedOnTrigger.gameObject.active = false;
                    }
                }
            }
            if (raiseSelf) {
                startY = transform.position.y;
                startX = transform.position.x;
            }
            if (canFire) {
                if (fireDirection.x > 0) {
                    sprite.SetSizeXY(-spriteSize, spriteSize);
                }
            }
        }
	}
	
	//
	void Update () {
        if (isButton) {
            RaycastHit hitCharacter, hitGround;
            
            
            Ray ray = new Ray(transform.position - Vector3.up*3, Vector3.up);

            
            if (Physics.Raycast(ray, out hitGround, 5, groundLayer)) {
                if (!switchedOn) {
                    Switch(true);
                    Sound.PlaySound(SoundType.Beep, 0.4f);
                }
            } else if (Physics.Raycast(ray, out hitCharacter, 5, playerLayer)) {
                if (!switchedOn) {
                    Switch(true);
                    Sound.PlaySound(SoundType.Beep, 0.4f);
                }
            } else {
                if (switchedOn) {
                    Switch(false);
                }
            }
        }
        if (!isTrigger) {
            float t = Time.deltaTime;
            counter += t;
            fireTime -= t;
            if (counter > frameTime) {
                counter -= frameTime;
                frame++;
                if (!disabled) {
                    Animate();
                } else {
                    Map.Smoke(transform.position + firePoint, 2);
                }
            }
            if (switchedOn && !disabled) {
                if (raiseSelf) {
                    
                    if (rigidbody != null) {
                        if (transform.position.y < startY + raiseAmount) {
                            if (raiseSelf) {
                           //     Debug.Log("SWITCHED ON " + (startY + raiseAmount) + "  " + transform.position.y);
                            }
                            rigidbody.AddForce(Vector3.up * 14000 * t);
                           // rigidbody.velocity *= 1 - t * 10;
                        }
                        if (fireTime <= 0) {
                            if (transform.position.y +2 >= startY + raiseAmount) {
                                if (target != null) {
                                    if (Mathf.Abs((transform.position.y + firePoint.y) - target.transform.position.y) < 3) {
                                        if (Mathf.Abs(target.transform.position.x - transform.position.x) < 50 && Mathf.Sign(target.transform.position.x - transform.position.x) == Mathf.Sign(fireDirection.x)) {
                                            frame = -2;
                                            fireTime = 1;
                                            fireY = transform.position.y;
                                            Animate();
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            if (raiseSelf) {
                if (fireTime > 0.2f) {
                    transform.position = new Vector3(startX, fireY, 0);
                } else {
                    transform.position = new Vector3(startX, transform.position.y, 0);
                }
                sprite.Transform();
            }
        }
	}
    void Animate() {
        if (canFire) {
            if (fireTime > 0 && !disabled) {
                if (raiseSelf) {
                    if (frame == 0) {
                        if (target.transform.position.y < transform.position.y + firePoint.y + 3) {
                            Shoot();
                        } else {
                            fireTime = 0;
                            Animate();
                        }
                    }
                }
                if (spriteLoop) {
                    int frameOffset = spriteFire + frame % spriteFireFrames;
                    sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                } else {
                    int frameOffset = spriteFire + Mathf.Clamp(frame, 0, spriteFireFrames);
                    sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                }
            } else {
                if (switchedOn) {                    
                    
                    if (spriteLoop) {
                        int frameOffset = spriteOn + frame % spriteOnFrames;
                        sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                    } else {
                        int frameOffset = spriteOn + Mathf.Clamp(frame, 0, spriteOnFrames);
                        sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                    }
                } else {
                    if (spriteLoop) {
                        int frameOffset = spriteOff + frame % spriteOffFrames;
                        sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                    } else {
                        int frameOffset = spriteOff + Mathf.Clamp(frame, 0, spriteOffFrames);
                        sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                    }
                }
            }
        } else {
            if (switchedOn && !disabled) {
                if (delaySwitch && frame == 5) {
                    
                    if (switchedOnTrigger != null) {
                        switchedOnTrigger.gameObject.active = true;
                    }
                }
                if (spriteLoop) {
                    int frameOffset = spriteOn + frame % spriteOnFrames;
                    sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                } else {
                    int frameOffset = spriteOn + Mathf.Clamp(frame, 0 , spriteOnFrames);
                    sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                }
            } else {
                if (spriteLoop) {
                    int frameOffset = spriteOff + frame % spriteOffFrames;
                    sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                } else {
                   
                    int frameOffset = spriteOff + Mathf.Clamp(frame, 0, spriteOffFrames);
                    sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((frameOffset) * spritePixels, (spriteRow + 1) * spritePixels);
                }
            }
        }
    }
    void Shoot() {
        Sound.PlaySound(SoundType.Shoot, 0.6f);
        fireTime = 0.6f;
        RaycastHit hitCharacter, hitGround;
       // float dist = Mathf.Abs(targetPosition.x - transform.position.x);
        Vector3 direction = fireDirection;
        Ray ray = new Ray(transform.position + firePoint- direction*5, direction);

        float characterXdiff = 1000;
        float groundXdiff = 1000;
        if (Physics.Raycast(ray, out hitGround, 200, groundLayer)) {
            //Debug.Log("HitGround");
            groundXdiff = Mathf.Abs(hitGround.collider.transform.position.x - transform.position.x);

        }

        if (Physics.Raycast(ray, out hitCharacter, 200, playerLayer)) {

            characterXdiff = Mathf.Abs(hitCharacter.collider.transform.position.x - transform.position.x);

        }
        if ((characterXdiff < 200 || groundXdiff < 200)) {
            if (characterXdiff < groundXdiff) {
                Collider other = hitCharacter.collider;
                Character otherCharacter = other.GetComponent(typeof(Character)) as Character;
                if (otherCharacter != null) {
                    //Debug.Log("Damage Player " + otherCharacter.gameObject.name);
                    otherCharacter.DamageLaser(direction);
                }
                if (otherCharacter == target) {                    
                   // Switch(false);
                   // DeactivateTurret();
                    target = null;
                }
                Map.DrawBeam(transform.position + firePoint, hitCharacter.point + direction * 2);
            } else {
                Collider other = hitGround.collider;
                Prop prop = other.GetComponent(typeof(Prop)) as Prop;
                if (prop != null) {
                    prop.Knock(ray.direction * 60);
                }
                Trap trap = other.GetComponent(typeof(Trap)) as Trap;
                if (trap != null) {
                    trap.Damage(direction);
                }
                Map.DrawBeam(transform.position + firePoint, hitGround.point + direction * 2);
            }
        } else {
           // Debug.Log("Shooting " + xDiff + " target " + targetPosition);
            Map.DrawBeam(transform.position + firePoint, transform.position + firePoint + direction * 150);
        }

    }
    public void Switch(bool on) {
        if (disableIfNoEnemies && Map.enemyCount > 0) {
           
        } else if ((enableIfNoEnemies && Map.enemyCount > 0) || !enableIfNoEnemies || !on) {
           
            if (on && !disabled) {
               
                frame = 0;
                switchedOn = true;
                if (!isTrigger) {
                    Animate();
                }
                if (!delaySwitch) {
                    if (switchedOnTrigger != null) {
                        if (isButton) {
                            //Debug.Log("Button Out ");
                            if (buttonSwitchOn) {
                                switchedOnTrigger.SendMessage("Switch", true, SendMessageOptions.DontRequireReceiver);
                                switchedOnTrigger.gameObject.active = true;
                            } else {
                                switchedOnTrigger.SendMessage("Switch", false, SendMessageOptions.DontRequireReceiver);
                                switchedOnTrigger.gameObject.active = false;
                            }
                        } else if (!isTrigger) {
                            switchedOnTrigger.gameObject.active = true;
                        } else {
                            switchedOnTrigger.SendMessage("Switch", true);
                        }
                    }
                } else {
                    frame = -15;
                }
            } else {
                if (delaySwitch) {
                   
                    frame = Mathf.Clamp(spriteOffFrames - frame, 0, spriteOffFrames);
                } else {
                    frame = 0;
                }
               
                switchedOn = false;
                if (!isTrigger) {
                    Animate();
                }
                if (switchedOnTrigger != null) {
                    if (isButton) {
                        //Debug.Log("Button Out ");
                        if (buttonSwitchOn) {
                            switchedOnTrigger.SendMessage("Switch", false, SendMessageOptions.DontRequireReceiver);
                            switchedOnTrigger.gameObject.active = false;
                        } else {
                            switchedOnTrigger.SendMessage("Switch", true, SendMessageOptions.DontRequireReceiver);
                            switchedOnTrigger.gameObject.active = true;
                        }
                    } else if (!isTrigger) {
                        switchedOnTrigger.gameObject.active = false;

                    } else {
                        switchedOnTrigger.SendMessage("Switch", false);                        
                    }
                    
                }
            }
        }
    }
    public void ActivateTurret(Character c) {
        if (!disabled) {
            if (!isTrigger) {
                switchedOn = true;
                target = c;
            } else {
                switchedOnTrigger.SendMessage("ActivateTurret", c);
            }
        }
    }
    public void DeactivateTurret() {
        if (!isTrigger) {
            switchedOn = false;
            target = null;
        } else {
            switchedOnTrigger.SendMessage("DeactivateTurret");   
        }
    }
    public void Damage(Vector3 direction) {
        disabled = true;
        switchedOn = false;
        DeactivateTurret();
        Animate();
        if (switchedOnTrigger != null) {
            switchedOnTrigger.gameObject.active = false;
        }
    }
}
