using UnityEngine;
using System.Collections;

public enum ButtonType {
    AwesomeShip,
    MissileShip,
    BasicShip,
    GunShip,
    ChangeName,
    Start,
    Menu,
    Exit,
    Quit,
    None
}

public class Button : MonoBehaviour {
    //
    public int spriteRow =0;
    public int spriteFrame =0;
    public int spritePixels = 16;
    //
    // ** this is public so that I can edit it from the inspector and see it from the Player class
    public ButtonType type = ButtonType.None;
    //
    SpriteManager spriteManager;
    Sprite sprite;
    //
   
    bool isDown = false;
    public bool isEnabled = true;
    public bool isSetup = false;

    // ** these functions are called from the Player class
    // ** they control the appearance of the button
    // ** All the Player class needs to know is which button it is over and what the Button.type variable is
    // ** the button itself doesn't do anything except hold a collider (which the Player raycasts against) and control a sprite
    //
    public void Over() {        
        if (!isDown) {
            sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame+1) * spritePixels, (spriteRow + 1) * spritePixels);           
        }
    }
    public void Out() {        
        if (isEnabled) {
            if (isDown) {
                sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame + 2) * spritePixels, (spriteRow + 1) * spritePixels);
            } else {
                sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame) * spritePixels, (spriteRow + 1) * spritePixels);
            }
        }
    }
    
    public void Down() {        
        isDown = true;
        sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame + 2) * spritePixels, (spriteRow + 1) * spritePixels);   
    }
    public void OtherDown() {
        if (isEnabled) {
            isDown = false;
            sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame) * spritePixels, (spriteRow + 1) * spritePixels);
        }
    }
    
    public void Enable(bool e) {     
        if (e) {
            sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame) * spritePixels, (spriteRow + 1) * spritePixels);               
        } else {
            sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame + 3) * spritePixels, (spriteRow + 1) * spritePixels);   
        }
        isEnabled = e;
    }
    void Awake() {
        GameObject uD = GameObject.Find("SpriteManagerButtons");
        spriteManager = uD.GetComponent(typeof(SpriteManager)) as SpriteManager;       
    }
    public void Setup(bool enabled) {
        if (!isSetup) {
            isSetup = true;
            sprite = spriteManager.AddSprite(gameObject, spritePixels, spritePixels, spriteFrame * spritePixels, (spriteRow + 1) * spritePixels, 16, 16, false);
            sprite.SetSizeXY(24, 24);
            sprite.lowerLeftUV = spriteManager.PixelCoordToUVCoord((spriteFrame) * spritePixels, (spriteRow + 1) * spritePixels);
            Enable(enabled);
        }
       
    }
	void Start () {
        Setup(true);
	}
    public void Move() {
        sprite.Transform();
    }
    public void Hide() {
        spriteManager.HideSprite(sprite);
    }
	
}
