using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public enum ExplosionType {
    Laser,
    LaserBig,
    Missle,
    PhotonTorpedo,
    GoodShip,
    EvilShip,
    DeathStar,
    None
}
public enum ShipType {
    AlliedBattleFortress,
    AlliedBattleShip,
    AlliedGunShip,
    AlliedMissileCarrier,
    AlliedCorvette,
    AlliedFighter,
    AxisBattleFortress,
    AxisBattleShip,
    AxisBomber,
    AxisBigBomber,
    AxisFighter,
    AxisGunShip,
    AxisDeathstar,
    None
}
public class LevelObject {
    public string name;
    public bool dead;
    public Vector3 position;
    public Vector3 rotation;
    public LevelObject(string name, bool dead, Vector3 position, Vector3 rotation) {
        this.name = name;
        this.dead = dead;
        this.position = position;
        this.rotation = rotation;
    }
}
public class LevelData {
    public List <LevelObject> characters;
    public List<LevelObject> props;
    public LevelData(){
        characters = new List<LevelObject>();
        props = new List<LevelObject>();
    }
}

public class Map : MonoBehaviour {
    static public List<Prop> props;
    static public List<Character> characters;
    static List<LevelData> levelData;
    static Map instance;
    //
    // ** All the particle emitters are public so that I can drag in their reference in the editor, I like doing things in the editor
    //
    public ParticleEmitter particlesStars;
    public ParticleEmitter particlesSparks;
    public ParticleEmitter particlesBubbles;
    public ParticleEmitter particlesSmoke;
    public ParticleEmitter particlesSmokePuffs;
    public ParticleEmitter particlesBubblesBig;
    public ParticleEmitter particlesElectric;
    public ParticleEmitter particlesElectricSpray;
    public ParticleEmitter particlesPlasma;
    public ParticleEmitter particlesPlasmaSpray;
    public ParticleEmitter particlesWarp;
    public ParticleEmitter particlesBloodPixel;
    public ParticleEmitter particlesWarpSparks;
    public ParticleEmitter particlesBeam;
    //
    // ** these I search for for some reason, these could have been public and dragged in
    // ** much of this code is copied and pasted from elsewhere
    SpriteManager spriteManagerEntities; 
    SpriteManager spriteManagerBlips; 
    //
    // ** These are all prefabs found in the project, you will see in the map prefab they're still assigned
    //
    public GameObject prefabShipAlliedBomber;
    public GameObject prefabShipAlliedFighter;
    public GameObject prefabShipAlliedGunShip;
    public GameObject prefabShipAlliedBattleShip;
    public GameObject prefabShipAlliedBattleFortress;
    public GameObject prefabShipAxisFighter;
    public GameObject prefabShipAxisBomber;
    public GameObject prefabShipAxisBigBomber;
    public GameObject prefabShipAxisGunShip;
    public GameObject prefabShipAxisBattleShip;
    public GameObject prefabShipAxisBattleFortress;
    public GameObject prefabShipAxisDeathstar;
    //
    public Beam prefabBeam;

    //
    public GameObject prefabBlip;
    //
    // ** I use a blank gameobject to hold the blips
    public Transform radar;
    //
    public GameObject prefabDebriGoodShip1;
    public GameObject prefabDebriGoodShip2;
    public GameObject prefabDebriEvilShip1;
    public GameObject prefabDebriEvilShip2;
    public GameObject prefabExplosionWisp;
    //
    // ** The counters for the enemy waves
    
    //
    public int nextLevel = 2;
    public int prevLevel = 1;
    public int elevatorLevel = 1;
    public float startSide = 0;
    static float playerStartSide = 0;
    static bool resetLevel = true;
    //
    // **
    static int levelNum = 1;
    static public int enemyCount = 0;
    //
    static LayerMask shipsLayer;
    //
    public bool checkFrameRate = false;
    public GUIText infoText;
    //
    public bool spawnStars = false;
    //
    static public bool failed = false;
    float failCountDown = 2;
    public static bool showingFailScreen = false;
    //
    void Awake() { // ** Awake occurs before Start ***
        Math.SetupLookupTables();
        instance = this;
        failed = false;
        showingFailScreen = false;
        props = new List<Prop>();
        characters = new List<Character>();
        if (levelData == null) {
            ResetLevelData();
        }
        if (startSide != 0 && resetLevel) {
            playerStartSide = startSide;
        }
        enemyCount = 0;
        //
        // ** you cannot use the SpriteManager during awake because it is using the Awake() function to set itself up and it's unpredictable which order things happen
        // ** but you can find it so as to use it later... (also it isn't actually unpredictable, it is based alpha numerically off of the name of the gameobject and the 
        // ** order the components are attached, which adds a lot of chaos which I'd rather not try organise)
        //
        GameObject uD = GameObject.Find("SpriteManagerEntities");
        spriteManagerEntities = uD.GetComponent(typeof(SpriteManager)) as SpriteManager; 
        //
        uD = GameObject.Find("SpriteManagerBlips");
        spriteManagerBlips = uD.GetComponent(typeof(SpriteManager)) as SpriteManager;
        //
        // ** I'm just doing this in case I am lame enough not to assign these values in the editor **
        //
        if (particlesBeam == null) {
            uD = GameObject.Find("ParticlesBeam");
            particlesBeam = uD.GetComponent(typeof(ParticleEmitter)) as ParticleEmitter;
        }

    }
    //

    void Start() {

        // 
        // ** Making loads of stars !! YAY !! You will see the back ones look faded, this is because I have fog on in the render settings, there are probably
        // ** More efficient ways of doing this (like changing the star's color eg:   particlesStars.Emit(position, Vector3.zero, 4, 4, new Color(1,1,1, distanceFromCamera));
        // ** In fact turning fog off may speed up this project a lot... Because there is no animator attached to the particleStars gameobject it will not override color assignment
        
       
        //
       
        //
        // ** start the music **
        //
       
       // Sound.PlayMusic(MusicType.Stage);

        Time.timeScale = 1; // ** this should be seting the speed to normal everytime you open up the gameplay scene
        Sound.AdjustPitch();
        //
        shipsLayer = 1 << 15;
        //
        if (spawnStars) {
            particlesStars.emit = true;
            particlesStars.transform.position = Vector3.up * 10 + Vector3.forward * 20;
            particlesStars.Emit(200);
        } else {
            particlesStars.emit = false;
        }
    }


    void Update() {
        float t = Time.deltaTime;
        //
        if (checkFrameRate) {
            if (infoText != null) {
                infoText.text = "framerate " + Mathf.Round(1 / Time.smoothDeltaTime * Time.timeScale);
            }
        }
        if (failed) {
            if (failCountDown > 0) {
                failCountDown -= t;
            } else {
                showingFailScreen = true;
            }

        }
        
      
	}
    static public void DrawBeam(Vector3 start, Vector3 end) {
        int length = (int)(Mathf.Abs(start.x - end.x)*3);
        instance.particlesBeam.rndVelocity = new Vector3(3, 3, 0);
        for (int i = 0; i < length; i++) {
            float p = i /(float)length;
            instance.particlesBeam.transform.position = start * (1 - p) + end * p;
            instance.particlesBeam.Emit(1);
        }
        Beam newBeam = Instantiate(instance.prefabBeam) as Beam;
        newBeam.Setup(start, end);
        instance.particlesBeam.rndVelocity = new Vector3(11, 11, 0);
        instance.particlesBeam.transform.position = end;
        instance.particlesBeam.Emit(25);
    }
    static public void ResetLevelData() {
        
        levelData = new List<LevelData>();
        for (int i = 0 ; i < 20; i++){

            levelData.Add(new LevelData());
        }
    }
    static void SaveLevelData() {
        
        levelData[levelNum] = new LevelData();
        for (int i = 0; i < characters.Count; i++) {
            levelData[levelNum].characters.Add(new LevelObject(characters[i].gameObject.name, characters[i].dead, characters[i].transform.position, characters[i].transform.eulerAngles));
        }
        for (int i = 0; i < props.Count; i++) {
            levelData[levelNum].props.Add(new LevelObject(props[i].gameObject.name, props[i].dead, props[i].transform.position, props[i].transform.eulerAngles));
        }
    }
    static public void ResetLevel() {
        resetLevel = true;

        Application.LoadLevel(levelNum);
    }
    static public bool PreviousLevel() {
        if (instance.prevLevel >= 0) {
            SaveLevelData();
            levelNum = instance.prevLevel;
            playerStartSide = 1;
        } else {
            return false;
        }
        resetLevel = false;
       
        Application.LoadLevel(levelNum);
        return true;
    }
    static public bool NextLevel() {
        if (instance.nextLevel >= 0) {
            SaveLevelData();
            levelNum = instance.nextLevel;
            playerStartSide = -1;
        } else {
            return false;
        }
        resetLevel = false;
       
        Application.LoadLevel(levelNum);
        return true;
    }
    static public bool ElevatorLevel() {
        if (instance.elevatorLevel >= 0) {
            SaveLevelData();
            levelNum = instance.elevatorLevel;
            playerStartSide = -1;
        } else {
            return false;
        }
        resetLevel = false;

        Application.LoadLevel(levelNum);
        return true;
    }
    static public void GotoLevel(int num) {
        SaveLevelData();
        levelNum = num;
        resetLevel = false;
       
        Application.LoadLevel(levelNum);
    }
    //
    // ** The functions, mostly static because it is easier to organise
    //
    static public void AddCharacter(Character character) {
        characters.Add(character);
        if (!character.evil) {
            Player.playerCharacter = character;
            character.transform.position = new Vector3(playerStartSide * 76, -14, 0);
        } else {
            enemyCount++;
            if (!resetLevel) {
                instance.AdjustCharacterToSavedData(character);
            }
        } 
    }
    static public void CharacterDeath(Character character) {
        if (character.evil) {
            enemyCount--;
            if (enemyCount <= 0) {
                Trap[] traps = FindObjectsOfType(typeof(Trap)) as Trap[];
                for (int i = 0; i < traps.Length; i++) {
                    if (traps[i].enableIfNoEnemies) {
                        traps[i].Switch(false);
                    }
                }
            }
        } else {
            failed = true;

        }
    }
    static public void SendAIToInvestigate(Vector3 position) {
        for (int i = 0; i < characters.Count; i++) {
            if (characters[i].evil) {
                if (Mathf.Abs(characters[i].transform.position.x - position.x) < 50) {
                    characters[i].SendMessage("Investigate", position, SendMessageOptions.DontRequireReceiver);
                }
            }
        }
    }
    static public void RemoveCharacter(Character character) {
        characters.Remove(character);
        
    }
    static public void AddProp(Prop prop) {
        props.Add(prop);
        if (!resetLevel ) {
            instance.AdjustPropToSavedData(prop);
        }        
    }
    static public void RemoveShip(Prop prop) {
        props.Remove(prop);
        
    }
    void AdjustCharacterToSavedData(Character character){
        if (levelData.Count > levelNum) {
            for (int i = 0; i < levelData[levelNum].characters.Count; i++) {
                if (levelData[levelNum].characters[i].name == character.gameObject.name) {
                    character.transform.position = levelData[levelNum].characters[i].position;
                    character.transform.eulerAngles = levelData[levelNum].characters[i].rotation;
                    character.dead = levelData[levelNum].characters[i].dead;
                }
            }
        }
    }
    void AdjustPropToSavedData(Prop prop) {
        Debug.Log("adjust prop " + prop.gameObject.name);
        if (levelData.Count > levelNum) {
            Debug.Log("level data! " + levelData[levelNum].props.Count);
            for (int i = 0; i < levelData[levelNum].props.Count; i++) {
                Debug.Log("adjust PROP " + levelData[levelNum].props[i].name);
                if (levelData[levelNum].props[i].name == prop.gameObject.name) {
                    prop.transform.position = levelData[levelNum].props[i].position;
                    prop.transform.eulerAngles = levelData[levelNum].props[i].rotation;
                    prop.dead = levelData[levelNum].props[i].dead;
                }
            }
        }
    }
    /*
    static public Transform GetNearestEnemy(Vector3 pos, bool isEvil, float maxRange) {
        int id = -1;
        float nearest = maxRange * maxRange;
        for (int i = 0; i < objects.Count; i++) {
            if (objects[i] != null) {
               // if (objects[i].evil != isEvil) {
                    float sqrDist = (objects[i].transform.position - pos).sqrMagnitude;
                    if (sqrDist < nearest) {
                        nearest = sqrDist;
                        id = i;
                    }
               // }
            }
        }
        if (id >= 0) {
            return objects[id].transform;
        } else {
            return null;
        }
    }
     */
    //
    // ** Blast (tm) technologies damage ships in a radius and push them around - I think my maths is very bad here but it seems to work
    //
    static public void Blast(Vector3 pos, float magnitude, float force) {
        float maxSqrDist = Mathf.Clamp(magnitude*magnitude/100, 0, 2500);
        for (int i = 0; i < props.Count; i++) {
            Vector3 diff = props[i].transform.position - pos;
            float sqrDist = (diff).sqrMagnitude;
            if (sqrDist < maxSqrDist) {
                // ** Knock seems to both damage and add a force to the ship's RigidBody
                props[i].SendMessage("Knock", diff * force);
                //ships[i].Knock(pos, Mathf.Clamp(magnitude / 10, 0, 50), force , Mathf.Clamp(magnitude / 2 * (1 - sqrDist / maxSqrDist), 0, 400));
            }
        }
    }
    /*
    static public void DamageArea(Vector3 pos, float damage, float range) {
        Collider[] shipColliders = Physics.OverlapSphere(pos, 3, shipsLayer);
        if (shipColliders.Length > 0) {
            
            for (int i = 0; i < shipColliders.Length; i++) {    
                // 
                // ** SendMessage is a sweet Unity function... Though it is actually slower than using "GetComponent(type(MyClass)) as MyClass" and then calling the function directly
                // ** But in this instance both the Gun class and the Ship class have a "public void Damage(float a)" function, so using SendMessage is better
                // ** Because otherwise we would be using two GetComponent()s
                // ** Also the code is more elegant (when you take away this huge chunk of commentary)
                shipColliders[i].SendMessage("Damage", damage, SendMessageOptions.DontRequireReceiver);                
            }
        }
    }
     */
    static public void Explosion(Vector3 pos, float magnitude, ExplosionType type) {
        //
        // ** My make a explosion function according to type... iy does what it says it does
        //
        if (type == ExplosionType.GoodShip || type == ExplosionType.EvilShip) {
            float a = Random.value * Mathf.PI;
            int dCount = Random.Range(2, 3);
            if (magnitude > 300) {
                if (magnitude > 600) {
                    dCount += 2;
                } else {
                    dCount += 1;
                }
            }
            // ** Chunks of ships **
            for (int i = 0; i < dCount; i++) {
                Vector3 dPos = Math.Point3OnCircle(a + Math.PI2 / dCount * i, 2);
                GameObject d;
                if (type == ExplosionType.GoodShip) { // ** pieces of good ship **
                    if (Random.value < 0.5f) {
                        d = Instantiate(instance.prefabDebriGoodShip1, pos + dPos * Random.value * 2, Quaternion.identity) as GameObject;
                    } else {
                        d = Instantiate(instance.prefabDebriGoodShip2, pos + dPos * Random.value * 2, Quaternion.identity) as GameObject;
                    }
                } else { // ** pieces of evil ship **
                    if (Random.value < 0.5f) {
                        d = Instantiate(instance.prefabDebriEvilShip1, pos + dPos * Random.value * 2, Quaternion.identity) as GameObject;
                    } else {
                        d = Instantiate(instance.prefabDebriEvilShip2, pos + dPos * Random.value * 2, Quaternion.identity) as GameObject;
                    }
                }
                d.SendMessage("Setup", dPos);
                //d.Setup(instance.spriteManagerEntities, instance.particlesSparks, 0.2f, 2, 5 + Random.value * 5, dPos+ dPos * Random.value * 12);
            }
            //
            // ** if you move an emitter to a position and ask it to emit then you get particles there
            //
            instance.particlesBubblesBig.transform.position = pos;
            instance.particlesBubblesBig.Emit((int)(magnitude/16));
            instance.particlesSmoke.transform.position = pos;
            instance.particlesSmoke.Emit((int)(magnitude / 10));
            instance.particlesBubbles.transform.position = pos;
            instance.particlesBubbles.Emit((int)(magnitude / 12));
            Blast(pos, magnitude, magnitude*10);
            //
            // The wisps are those sweet traily bits of the explosion which you get in all good special effects
            //
            int wisps = Random.Range(2, 4);
            if (magnitude > 500) {
                wisps += 3;
            }
            for (int i = 0; i < wisps; i++) {                
                a += Mathf.PI;
                Vector3 dPos = Math.Point3OnCircle(a + Math.PI2 / wisps * i, 2);
                GameObject wisp = Instantiate(instance.prefabExplosionWisp, pos + dPos * Random.value * 2, Quaternion.identity) as GameObject;
                wisp.SendMessage("Setup", dPos);
                //wisp.Setup(instance.particlesSmokePuffs, 0.1f, 2, 2, dPos * 27, 1f);
            }
            Sound.PlaySound(SoundType.Explode, 0.6f);
        } else if (type == ExplosionType.Laser) {
            instance.particlesElectric.transform.position = pos;
            instance.particlesElectric.Emit(12);
            instance.particlesElectricSpray.transform.position = pos;
            instance.particlesElectricSpray.Emit(6);
            instance.particlesSmokePuffs.transform.position = pos;
            instance.particlesSmokePuffs.Emit(24);
            Sound.PlaySound(SoundType.Hit, 0.2f);
        } else if (type == ExplosionType.LaserBig) {
            instance.particlesElectric.transform.position = pos;
            instance.particlesElectric.Emit(12);
            instance.particlesElectricSpray.transform.position = pos;
            instance.particlesElectricSpray.Emit(36);
            instance.particlesSmokePuffs.transform.position = pos;
            instance.particlesSmokePuffs.Emit(24);
            Sound.PlaySound(SoundType.Hit, 0.3f);
        } else if (type == ExplosionType.Missle) {
            instance.particlesPlasma.transform.position = pos;
            instance.particlesPlasma.Emit(12);
            instance.particlesPlasmaSpray.transform.position = pos;
            instance.particlesPlasmaSpray.Emit(12);
            instance.particlesSmokePuffs.transform.position = pos;
            instance.particlesSmokePuffs.Emit(32);
            Sound.PlaySound(SoundType.Hit, 0.4f);
        } else if (type == ExplosionType.PhotonTorpedo) {
            instance.particlesPlasma.transform.position = pos;
            instance.particlesPlasma.Emit(12);
            instance.particlesPlasmaSpray.transform.position = pos;
            instance.particlesPlasmaSpray.Emit(32);
            instance.particlesSmokePuffs.transform.position = pos;
            instance.particlesSmokePuffs.Emit(32);
            instance.particlesBubblesBig.transform.position = pos;
            instance.particlesBubblesBig.Emit(6);
            Blast(pos, 400, 4000);
            Sound.PlaySound(SoundType.Explode, 0.4f);
        } else if (type == ExplosionType.DeathStar) {
            instance.particlesSparks.transform.position = pos;
            instance.particlesSparks.Emit(32);
            instance.particlesBubblesBig.transform.position = pos;
            instance.particlesBubblesBig.Emit(40);
            instance.particlesPlasmaSpray.transform.position = pos;
            instance.particlesPlasmaSpray.Emit(60);
            
            Sound.PlaySound(SoundType.Explode, 1.1f);
        }
    }
    //
    // ** these functions just get the emitters to make particles 
    //
    // ** if you move an emitter to a position and ask it to emit then you get particles there
    //
    static public void Sparks(Vector3 pos, float amount) {
        instance.particlesSparks.transform.position = pos;
        instance.particlesSparks.Emit((int)amount);
    }
    static public void Beam(Vector3 pos, float amount) {
        instance.particlesBeam.transform.position = pos;
        instance.particlesBeam.Emit((int)amount);
    }
    static public void FireSparks(Vector3 pos, float amount) {
        instance.particlesPlasma.transform.position = pos;
        instance.particlesPlasma.Emit((int)amount);
    }
    static public void Smoke(Vector3 pos, float amount) {
        instance.particlesSmokePuffs.transform.position = pos;
        instance.particlesSmokePuffs.Emit((int)amount);
    }
    static public void SmokeBig(Vector3 pos, float amount) {
        instance.particlesSmoke.transform.position = pos;
        instance.particlesSmoke.Emit((int)amount);
    }
    static public void Particles(Vector3 pos, float amount) {
        instance.particlesPlasmaSpray.transform.position = pos;
        instance.particlesPlasmaSpray.Emit((int)amount);
    }
    static public void Electric(Vector3 pos, float amount) {
        instance.particlesElectric.transform.position = pos;
        instance.particlesElectric.Emit((int)amount);
    }
    static public void Warp(Vector3 pos, int amount) {
        instance.particlesWarp.transform.position = pos;
        instance.particlesWarp.Emit(amount);
       
    }
    static public void Blood(Vector3 pos, int amount) {
        instance.particlesWarp.transform.position = pos;
        instance.particlesWarp.Emit(amount);

    }
    static public void BloodPixels(Vector3 pos, int amount) {
        instance.particlesBloodPixel.transform.position = pos;
        instance.particlesBloodPixel.Emit(amount);

    }
    static public void WarpSparks(Vector3 pos, int amount) {
        instance.particlesWarpSparks.transform.position = pos;
        instance.particlesWarpSparks.Emit(amount);
    }
    static public void Bubbles(Vector3 pos, int amount) {
        instance.particlesBubbles.transform.position = pos;
        instance.particlesBubbles.Emit(amount);
    }
    static public void BubblesBig(Vector3 pos, int amount) {
        instance.particlesBubblesBig.transform.position = pos;
        instance.particlesBubblesBig.Emit(amount);
    }
    /*
    static public Transform GetRandomShip(bool evil) {
        // ** This is called by the enemy ships to determine who to follow **
        // ** I should probably just make a List of allied ships and a list of Axis ships rather than just itterating through the whole list every time
        //
        Transform chosenShip = null;
        for (int i = 0; i < objects.Count; i++) {
            if (objects[i] != null) {
                //if (ships[i].evil == evil) {
                if (chosenShip == null || Random.value < 1f / objects.Count) {
                    chosenShip = objects[i].transform;
                }
                // }
            }
        }
        return chosenShip;
    }
    static public Vector3 GetAverageAlliedPosition() {
        // ** Once again an Allied list would be better ***
        // ** This function is called to determine where to spawn in a new ship
        //
        Vector3 averagePosition = Vector3.zero;
        int goodShips = 0;
        for (int i = 0; i < objects.Count; i++) {
            //   if (!ships[i].evil) {
            averagePosition += objects[i].transform.position;
            goodShips++;
            // }
        }
        if (goodShips > 0) {
            averagePosition /= goodShips;
        }
        return averagePosition;
    }
     */
    /*
    static public GameObject GetNewBlip() {
        // ** Returns an instance of the blip prefab
        // ** I like Instantiating things from a central class (Like Map) though the ship class could just as easily have contained the variable for the Wisp Prefab
        GameObject newBlip = Instantiate(instance.prefabBlip) as GameObject;
        newBlip.transform.parent = instance.radar.transform;
        return newBlip;
    }
     */
    static public void SpawnShip(ShipType type, Vector3 pos) {
        //
        // ** Spawns a ship of your choice where you want it
        // ** Even makes those purply particle things
        // ** doesn't play the sound however (because allot of spawnships may happen on one frame
        //
        if (type == ShipType.AlliedBattleShip) {
            GameObject newShip = Instantiate(instance.prefabShipAlliedBattleShip, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
         //   newShip.Setup(instance.spriteManagerEntities, instance.spriteManagerBlips);
            Warp(pos, 30);
        } else if (type == ShipType.AlliedMissileCarrier) {
            GameObject newShip = Instantiate(instance.prefabShipAlliedBomber, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 20);
        } else if (type == ShipType.AlliedGunShip) {
            GameObject newShip = Instantiate(instance.prefabShipAlliedGunShip, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 20);
        } else if (type == ShipType.AlliedFighter) {
            GameObject newShip = Instantiate(instance.prefabShipAlliedFighter, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 15);
        } else if (type == ShipType.AxisFighter) {
            GameObject newShip = Instantiate(instance.prefabShipAxisFighter, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 14);
        } else if (type == ShipType.AxisGunShip) {
            GameObject newShip = Instantiate(instance.prefabShipAxisGunShip, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 20);
        } else if (type == ShipType.AxisBomber) {
            GameObject newShip = Instantiate(instance.prefabShipAxisBomber, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 20);
        } else if (type == ShipType.AxisBigBomber) {
            GameObject newShip = Instantiate(instance.prefabShipAxisBigBomber, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 27);
        } else if (type == ShipType.AxisBattleShip) {
            GameObject newShip = Instantiate(instance.prefabShipAxisBattleShip, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 32);
        } else if (type == ShipType.AxisBattleFortress) {
            GameObject newShip = Instantiate(instance.prefabShipAxisBattleFortress, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 50);
        } else if (type == ShipType.AxisDeathstar) {
            GameObject newShip = Instantiate(instance.prefabShipAxisDeathstar, pos, Quaternion.identity) as GameObject;
            newShip.SendMessage("Setup");
            Warp(pos, 70);
        }

    }
    void SpawnEnemies(int danger, Vector3 pos) {
        //
        // ** This is called whenever WaveCounter reaches WaveRate... This function determines who is coming at you
        //
        if (danger < 1) {
            float xOff = Random.value * 6;
            float yOff = 6 - xOff;
            SpawnShip(ShipType.AxisFighter, pos + new Vector3(xOff, yOff));
            SpawnShip(ShipType.AxisFighter, pos + new Vector3(-yOff, -xOff));
        } else if (danger < 2) {            
            float xOff = Random.value * 7;
            float yOff = 6 - xOff;
            SpawnShip(ShipType.AxisFighter, pos + new Vector3(xOff, yOff));
            SpawnShip(ShipType.AxisFighter, pos + new Vector3(-yOff, -xOff));
            SpawnShip(ShipType.AxisFighter, pos + new Vector3(yOff, xOff));
        } else if (danger < 3) {
            float xOff = Random.value * 8;
            float yOff = 6 - xOff;
            SpawnShip(ShipType.AxisGunShip, pos + new Vector3(xOff, yOff));
            SpawnShip(ShipType.AxisGunShip, pos + new Vector3(-yOff, -xOff));
            SpawnShip(ShipType.AxisGunShip, pos + new Vector3(yOff, xOff));
        } else if (danger < 4) {
            float xOff = Random.value * 9;
            float yOff = 6 - xOff;
            SpawnShip(ShipType.AxisBomber, pos + new Vector3(xOff, yOff));
            SpawnShip(ShipType.AxisBigBomber, pos + new Vector3(-yOff, -xOff));
            SpawnShip(ShipType.AxisFighter, pos + new Vector3(yOff, xOff));
            SpawnShip(ShipType.AxisFighter, pos + new Vector3(-xOff, -yOff));
        } else if (danger < 5) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 12);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2*0.2f, 12);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.4f, 12);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.6f, 12);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.8f, 12);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
        } else if (danger < 6) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 12);            
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.33f, 12);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.7f, 12);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);            
        } else if (danger < 7) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.08f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.16f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.24f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.32f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.40f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.48f, 60);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.56f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.64f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.72f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.80f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.86f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.94f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
        } else if (danger < 8) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 18);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.2f, 18);
            SpawnShip(ShipType.AxisGunShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.4f, 18);
            SpawnShip(ShipType.AxisGunShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.6f, 18);
            SpawnShip(ShipType.AxisGunShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.8f, 18);
            SpawnShip(ShipType.AxisGunShip, pos + offSet);
        } else if (danger < 9) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 24);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.125f, 36);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.25f, 36);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.375f, 36);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.5f, 36);
            SpawnShip(ShipType.AxisDeathstar, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.625f, 36);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.75f, 36);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.875f, 36);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
           
        } else if (danger < 10) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 24);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.25f, 50);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.5f, 50);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.75f, 50);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);            
        } else if (danger < 12) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 50);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.08f, 50);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.16f, 50);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.24f, 50);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.32f, 50);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.40f, 50);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.48f, 50);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.56f, 50);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.64f, 50);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.72f, 50);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.80f, 50);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.86f, 50);
            SpawnShip(ShipType.AxisBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.94f, 50);
            SpawnShip(ShipType.AxisBomber, pos + offSet);
        } else if (danger >= 12) {
            float a = Math.PI2 * Random.value;
            Vector3 offSet = Math.Point3OnCircle(a, 60);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.08f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.16f, 60);
            SpawnShip(ShipType.AxisDeathstar, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.24f, 60);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.32f, 60);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.40f, 60);
            SpawnShip(ShipType.AxisBigBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.48f, 60);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.56f, 60);
            SpawnShip(ShipType.AxisBattleFortress, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.64f, 60);
            SpawnShip(ShipType.AxisBattleShip, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.72f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.80f, 60);
            SpawnShip(ShipType.AxisFighter, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.86f, 60);
            SpawnShip(ShipType.AxisBomber, pos + offSet);
            offSet = Math.Point3OnCircle(a + Math.PI2 * 0.94f, 60);
            SpawnShip(ShipType.AxisBomber, pos + offSet);
        }
        Sound.PlaySound(SoundType.Sexy, 0.5f);
    }
}
