using UnityEngine;
using System.Collections;

public enum MusicType {
    Menu,
    Stage,
    Win,
    None
}
public enum SoundType {
    Hit,
    No,
    Shoot,
    Laser,
    Missile,
    Explode,
    Beep,
    Sexy,
    End,
    Charge,
    None
}

public class Sound : MonoBehaviour {

    // ** All of these are public so that I can edit them in the inspector in Unity
    // ** have a look at the Sound Prefab in prefabs, you should see all these variables are set
    //
    public AudioClip effectHit1;
    public AudioClip effectHit2;
    public AudioClip effectHit3;
    public AudioClip effectShoot1;
    public AudioClip effectShoot2;
    public AudioClip effectLaser1;
    public AudioClip effectLaser2;
    public AudioClip effectExplode1;
    public AudioClip effectExplode2;
    public AudioClip effectExplode3;   
    public AudioClip effectMissile1;
    public AudioClip effectMissile2;    
    public AudioClip effectBeep1;
    public AudioClip effectBeep2;
    public AudioClip effectBeep3;
    public AudioClip effectBeep4;
    public AudioClip effectSexy1;
    public AudioClip effectNo1;
    public AudioClip effectEnd1;
    public AudioClip effectEnd2;
    public AudioClip effectCharge1;
    //
    public AudioClip musicMenu;
    public AudioClip musicStage;
	//
	AudioSource unitSounds;   
    AudioSource music;
	AudioSource effectsSounds;
    //
    static MusicType levelMusicType = MusicType.None;
	//
	static Sound sound;
	//
    bool isSetup = false;
    static bool soundOn = true;
    static public bool IsOn() {
        return soundOn;
    }
    static public void SoundOn() {
        PlayerPrefs.SetInt("soundon", 2);
        soundOn = true;
        PlayMusic(levelMusicType);
    }
    static public void SoundOff() {
        PlayerPrefs.SetInt("soundon", 1);
        soundOn = false;
        StopAllMusic();
    }
    public void Setup() {
        if (!isSetup) {
            //
            // ** the Sound gameobject needs some audiosources in order to play the audio clips
            //
            sound = this;
            //
            if (GetComponent(typeof(AudioSource)) == null) {
                unitSounds = gameObject.AddComponent(typeof(AudioSource)) as AudioSource;
                unitSounds.rolloffFactor = 0;
                music = gameObject.AddComponent(typeof(AudioSource)) as AudioSource;
                music.rolloffFactor = 0;
                effectsSounds = gameObject.AddComponent(typeof(AudioSource)) as AudioSource;
                effectsSounds.rolloffFactor = 0;
                //
                // 
                gameObject.name = "Sound (Indestructible)";
                // ** This call here makes sure Application.LoadLevel calls don't destroy this object
                DontDestroyOnLoad(gameObject);
                //
                // ** Checks to see whether sounds were on previously
                //
                int tempSoundIsOn = PlayerPrefs.GetInt("soundon");
                if (tempSoundIsOn == 1) {
                    soundOn = false;
                }                
            }
            //            
            //
            isSetup = true;
        }
    }
	void Awake () {
        // ** there is a sound object in every scene so that the game can be tested in every scene
        // ** this Awake() call checks whether there is already a Sound object, and if so this object Destroys itself
        //
        if (FindObjectsOfType(typeof(Sound)).Length <= 1 && sound == null) {
            Setup();
        } else {            
            Destroy(gameObject);
        }
	}   
    static public void StopAllMusic() {
        sound.music.Stop();
    }
    static public void PlayMusic(MusicType type) {
        //
        // ** This plays either the stage music or the menu music
        // ** this uses the AudioSource.Play function becuse it needs to loop
        //
        Debug.Log("music type " + levelMusicType);
        if (soundOn) {
            
            if (type == MusicType.Menu && levelMusicType != MusicType.Menu) {
                sound.music.Stop();
                sound.music.volume = 0.75f;
                sound.music.loop = true;
                sound.music.clip = sound.musicMenu;
                sound.music.Play();
            } else if (type == MusicType.Stage && levelMusicType != MusicType.Stage) {
                sound.music.Stop();
                sound.music.volume = 0.4f;
                sound.music.loop = true;
                sound.music.clip = sound.musicStage;
                sound.music.Play();
            }
        }
        levelMusicType = type;
    }
    static public void AdjustPitch() {
        float p = Mathf.Sqrt(Time.timeScale);
        sound.music.pitch = p;
        sound.effectsSounds.pitch = p;
        sound.unitSounds.pitch = p;        
    }
    static public void RandomPitch() {
        float r = 0.6f + Random.value * 0.8f;
        sound.effectsSounds.pitch = r;
        sound.unitSounds.pitch = r;       
    }
    static public void PlaySound(SoundType type, float v) {        
        if (v >0f && soundOn) {
            switch (type) {
                case SoundType.Charge:
                    sound.effectsSounds.PlayOneShot(sound.effectCharge1, v);
                    break;
                case SoundType.Sexy:
                    sound.effectsSounds.PlayOneShot(sound.effectSexy1, v);
                    break;
                case SoundType.No:
                    sound.effectsSounds.PlayOneShot(sound.effectNo1, v);
                    break;    
                case SoundType.Explode:
                    switch (Random.Range(0, 3)) {
                        case 1:
                            sound.effectsSounds.PlayOneShot(sound.effectExplode1, v);
                            break;
                        case 2:
                            sound.effectsSounds.PlayOneShot(sound.effectExplode2, v);
                            break;
                        default:
                            sound.effectsSounds.PlayOneShot(sound.effectExplode3, v);
                            break;
                    }
                    break;
                case SoundType.End:
                    switch (Random.Range(0, 2)) {
                        case 1:
                            sound.unitSounds.PlayOneShot(sound.effectEnd1, v);
                            break;
                        default:
                            sound.unitSounds.PlayOneShot(sound.effectEnd2, v);
                            break;
                    }
                    break;
                case SoundType.Shoot:
                    switch (Random.Range(0, 2)) {
                        case 1:
                            sound.unitSounds.PlayOneShot(sound.effectShoot1, v);
                            break;
                        default:
                            sound.unitSounds.PlayOneShot(sound.effectShoot2, v);
                            break;
                    }
                    break;
                case SoundType.Hit:
                    switch (Random.Range(0, 3)) {
                        case 1:
                            sound.effectsSounds.PlayOneShot(sound.effectHit1, v);
                            break;
                        case 2:
                            sound.effectsSounds.PlayOneShot(sound.effectHit2, v);
                            break;
                        default:
                            sound.effectsSounds.PlayOneShot(sound.effectHit3, v);
                            break;
                    }
                    break;
                case SoundType.Laser:
                    switch (Random.Range(0, 2)) {
                        case 1:
                            sound.unitSounds.PlayOneShot(sound.effectLaser1, v);
                            break;
                        default:
                            sound.unitSounds.PlayOneShot(sound.effectLaser2, v);
                            break;
                    }
                    break;
              
                case SoundType.Beep:
                    switch (Random.Range(0, 4)) {
                        case 1:
                            sound.effectsSounds.PlayOneShot(sound.effectBeep1, v);
                            break;
                        case 2:
                            sound.effectsSounds.PlayOneShot(sound.effectBeep2, v);
                            break;
                        case 3:
                            sound.effectsSounds.PlayOneShot(sound.effectBeep3, v);
                            break;

                        default:
                            sound.effectsSounds.PlayOneShot(sound.effectBeep4, v);
                            break;
                    }
                    break;
                case SoundType.Missile:
                    switch (Random.Range(0, 2)) {
                        case 1:
                            sound.unitSounds.PlayOneShot(sound.effectMissile1, v);
                            break;

                        default:
                            sound.unitSounds.PlayOneShot(sound.effectMissile2, v);
                            break;
                    }
                    break;
                default:
                    break;

            }
        }
    }
}
