using UnityEngine;
using System.Collections;
using System.Collections.Generic;
//
public enum ActionType {
    Idle,
    Walk,
    Attack,
    Fall,
    Die,
    DieLaser,
    DieFire,
    Slide,
    Jump,
    Float,
    None
}
public struct Action {
    public ActionType type;
    public Vector3 position;
    public Vector3 direction;
    public Action(ActionType type, Vector3 position, Vector3 direction) {
        this.type = type;

        this.position = position;
        this.direction = direction;
    }
}

public class Player : MonoBehaviour {
    // ** Okay, this is where it happens, user input is collected here
    // ** Also, if you look in the Hierarchy in Unity you will see that the Cameras are all parented to the Player GameObject
    // ** This means that if the transform of player is moved then all the cameras move with it
    // ** i.e. you can scroll around
    // ** Also all the menus are parented to the Player GameObject as well
    //
    bool up, down, left, right = false; // ** define whether it should scroll
    //
    public GUIText scoreText;
    
    static int score = 0;
    float scoreShow = 0;
    public GUIText tooltipText;
    public Material tooltipMaterial;
    //
    static Button overButton = null;
    static Button downButton = null;
    static Button disabledButton = null;
    LayerMask menuLayer, shipsLayer;
    GameObject selectedShip;
    bool dragging = false;
    // ** the Line for commanding ships is assigned in the editor
    public LineRenderer draggingLine;
    //
    // ** these styles are modified in the editor
    public GUIContent styleButtonContent;
    public GUIStyle styleButtonExit;
    public GUIStyle styleButtonPause;
    public Texture textureConsole;
    public Texture textureFail;
    public GUIStyle styleLogo;
    public GUIStyle styleConsoleString;
    public GUIStyle styleConsoleInput;
    //
   static  public Character playerCharacter;
   static List<string> commandsList;
   static int commandNumber = 0;
    //
   public Fire prefabFire;
    //
    static public bool canUseElevator = false;
    //
    //
    // ** the mainCamera is assigned in the editor... I think it is slightly faster than using Camera.mainCamera .. and more predictable
    public Camera mainCamera;
    static Button[] buttons;
    //
    bool consoleOn = false;
    int consoleHeight = 0;
    string consoleString = "";

    void Start() { // ** Start occurs once just before the first frame the gameObject is active **
        if (commandsList == null) {
            commandsList = new List<string>();
        }
        commandNumber = commandsList.Count ;
        tooltipText = GameObject.Find("Tooltip").GetComponent(typeof(GUIText)) as GUIText;
      //  scoreText = GameObject.Find("Score").GetComponent(typeof(GUIText)) as GUIText;
      //  scoreText.pixelOffset = new Vector2(Screen.width / 2, Screen.height - 10);
        tooltipText.pixelOffset = new Vector2(Screen.width / 2, 110);
        //
        menuLayer = 1 << 16; // the LayerMasks are actually bit masks        
        shipsLayer = 1 << 15;
        //
        //
        draggingLine.SetVertexCount(2);
        draggingLine.enabled = false;
        Vector3 buttonYPos = Camera.mainCamera.ScreenToWorldPoint(new Vector3(0, 40, 120));
        buttons = FindObjectsOfType(typeof(Button)) as Button[];
        for (int i = 0; i < buttons.Length; i++) {
            buttons[i].transform.position = new Vector3(buttons[i].transform.position.x, buttonYPos.y, buttons[i].transform.position.z);
            if (!buttons[i].isSetup) {
                buttons[i].Setup(true);
            }
            buttons[i].Move();
        }
        //
        score = 0;
        CheckIfButtonsAreEnabled();
       // Map.SpawnShip(ShipType.AlliedFighter, Vector3.zero);
        Sound.PlaySound(SoundType.Sexy, 0.1f);
        
    }
    void OnGUI() {
        // ** This draws the pause button and the exit button... I got lazy, they could have been my Button class and I would have saved 2 drawcalls, but the exit button was a funny shape      
        //
        /*
        if (GUI.Button(new Rect(4, Screen.height - 68, 96, 96), styleButtonContent, styleButtonExit)) {
            // ** exit the level
            //
            if (MainMenu.nameText != "") {
                // ** save the score
                MainMenu.InsertScore(MainMenu.nameText, score);
            }
            Time.timeScale = 1;
            Sound.PlaySound(SoundType.Explode, 0.2f);
            // ** Go to the menu level (the zero refers to the index of the level in the publish settings **
            Application.LoadLevel(0);
        } else if (GUI.Button(new Rect(Screen.width - 36, Screen.height - 36, 32, 32), styleButtonContent, styleButtonPause)) {
            Pause();
        } else {
         */
        if (Map.showingFailScreen) {
            GUI.Box(new Rect(0, -150, 800, 800), textureFail, styleLogo);
        } else {
            if (consoleOn) {
                if (consoleHeight < 100) {
                    consoleHeight += 3;
                } else {
                    consoleHeight = 100;
                }

                GUI.Box(new Rect(Screen.width / 2 - 512, 600 - consoleHeight, 1024, 1024), textureConsole, styleLogo);
                GUI.SetNextControlName("ConsoleField");
                Rect nameRect = new Rect(24, Screen.height - consoleHeight + 8, 570, 34);
                consoleString = GUI.TextArea(nameRect, consoleString, 24, styleConsoleInput);
                Rect textRect = new Rect(24, Screen.height - consoleHeight + 46, 770, 24);
                GUI.TextArea(textRect, "Commands: Push, Swap, Attack, Burn", 56, styleConsoleInput);
                textRect = new Rect(24, Screen.height - consoleHeight + 70, 770, 24);
                GUI.TextArea(textRect, "Example: Robot1 Attack... or... Push PotPlant", 56, styleConsoleInput);
                GUI.FocusControl("ConsoleField");
                for (int i = 0; i < Map.props.Count; i++) {

                    Vector3 propPosition = Camera.main.WorldToScreenPoint(Map.props[i].transform.position);
                    Rect propRect = new Rect(propPosition.x - 75, Screen.height - propPosition.y - 12 - Map.props[i].spriteSize * 2, 150, 24);
                    GUI.TextArea(propRect, Map.props[i].gameObject.name, 24, styleConsoleString);
                }
                for (int i = 0; i < Map.characters.Count; i++) {
                    if (Map.characters[i].gameObject.name != "PlayerCharacter") {
                        Vector3 propPosition = Camera.main.WorldToScreenPoint(Map.characters[i].transform.position);
                        Rect propRect = new Rect(propPosition.x - 75, Screen.height - propPosition.y - 12 - Map.characters[i].spriteSize * 3, 150, 24);
                        GUI.TextArea(propRect, Map.characters[i].gameObject.name, 24, styleConsoleString);
                    }
                }
            }
        }
        /*
        }
         */


    }
	void Update () {
        
        //
       
        
        if (!consoleOn) {
            GetInput();
            ControlCharacter();
        } else {
            GetConsoleInput();
        }
        //
        //ShowScore();
	}
    void PressTilda(bool on) {
        if (!Map.failed) {
            up = down = right = left = false;
            consoleString = "";

            consoleOn = on;
            consoleHeight = 0;

            if (on) {
                playerCharacter.GotoLaptop();
                Time.timeScale = 0;
            } else {
                playerCharacter.Animate();
                Time.timeScale = 1;
            }
        } else {
            Map.ResetLevel();
        }
    }
    void ShowScore() {
        // ** shows the score 
        if (scoreShow < score) {
            scoreShow = Mathf.Ceil(Mathf.Lerp(scoreShow, score, Time.deltaTime * 2));
        } else {
            scoreShow = Mathf.Floor(Mathf.Lerp(scoreShow, score, Time.deltaTime * 2));
        }
        //scoreText.text = "SCORE " + scoreShow;       
    }
    static public void AddScore(int s) {
        score += s;
        CheckIfButtonsAreEnabled();
    }
    void ControlCharacter() {
        // ** the up, down, left, right bools are set in the GetInput function
        if (playerCharacter != null) {
            if (up) {
                playerCharacter.Jump();
                //    transform.position += Vector3.up * t * 80 /Time.timeScale;
            }
            if (down) {
                //    transform.position -= Vector3.up * t * 80 / Time.timeScale;
            }
            if (left) {
                playerCharacter.Walk(new Vector3(-40,0,0));
            }
            if (right) {
                playerCharacter.Walk(new Vector3(40, 0, 0));
            }
            if (!left && !right) {
                playerCharacter.Stop();
            }
        }
    }
    void GetConsoleInput() {
        // ** Input.GetKeyDown(KeyCode) is a lovely Unity function, works like you'd want it too, though there is no persistent (every frame) key check that I know of
        
        if (Input.GetButtonDown("Fire2")) {
            PressTilda(false);
        }
        
        if (Input.GetKeyDown(KeyCode.Return)) {
            if (consoleString.ToLower().Contains("reset")) {
                Map.ResetLevel();
                return ;
            }
            commandsList.Add(consoleString);
            commandNumber = commandsList.Count;
            for (int i = 0; i < Map.props.Count; i++) {
                if (consoleString.ToLower().Contains(Map.props[i].gameObject.name.ToLower())) {
                    GiveCommand(Map.props[i].gameObject, consoleString);
                }
            } 
            for (int i = 0; i < Map.characters.Count; i++) {
                if (consoleString.ToLower().Contains(Map.characters[i].gameObject.name.ToLower())) {
                    GiveCommand(Map.characters[i].gameObject, consoleString);
                }
            }
            PressTilda(false);
        }
        if (Input.GetKeyDown(KeyCode.UpArrow)) {
            if (commandNumber > 0 && commandsList.Count > 0) {
                commandNumber--;
                consoleString = commandsList[commandNumber];
            }
        }
    }
    void GiveCommand(GameObject p, string commandString) {
        if (commandString.ToLower().Contains("knock") || commandString.ToLower().Contains("push")) {
           p.SendMessage("Knock", (-playerCharacter.transform.position + p.transform.position).normalized * 90);

        } else if (commandString.ToLower().Contains("swap")) {
            //p.SendMessage("Knock", (Vector3.right + Vector3.up) * 15);
            Vector3 transferPosition = new Vector3(p.transform.position.x , p.transform.position.y, 0);
            p.transform.position = new Vector3(playerCharacter.transform.position.x, playerCharacter.transform.position.y, p.transform.position.z);
            p.SendMessage("ResetPosition", SendMessageOptions.DontRequireReceiver);
            playerCharacter.transform.position = transferPosition;
            Map.Electric(p.transform.position, 40);
            Map.Electric(playerCharacter.transform.position, 40);
        } else if (commandString.ToLower().Contains("attack")) {
            //p.SendMessage("Knock", (Vector3.right + Vector3.up) * 15);
            Vector3 direction = Vector3.right * Mathf.Sign(p.rigidbody.velocity.x);
            p.SendMessage("Attack", p.transform.position +direction*50);
        } else if (commandString.ToLower().Contains("burn")) {
           
            if (commandString.ToLower().Contains("potplant")) {
                Fire newFire = Instantiate(prefabFire, p.transform.position, Quaternion.identity) as Fire;
                newFire.time = 3 + Random.value * 2;
                newFire.transform.parent = p.transform;
            } else {
                float a = Random.value * 4;
                for (int i = 0; i < 2; i++) {
                    Vector3 diff = Math.Point3OnCircle(a + (float)i * Mathf.PI, 4);
                    Fire newFire = Instantiate(prefabFire, p.transform.position + diff, Quaternion.identity) as Fire;
                    newFire.time = 3 + Random.value * 2;
                    newFire.transform.parent = p.transform;
                }
            }
            Map.SendAIToInvestigate(p.transform.position);
        }
    }
    void GetInput() {
        // ** Input.GetKeyDown(KeyCode) is a lovely Unity function, works like you'd want it too, though there is no persistent (every frame) key check that I know of
        if (Input.GetKeyDown("space")) {
            if (canUseElevator && !playerCharacter.dead) {
                Map.ElevatorLevel();
            }
        }
        if (Input.GetButtonDown("Fire2")) {
            PressTilda(true);
        }
        
        if (Input.GetKeyDown(KeyCode.KeypadPlus)) {
            Time.timeScale = Mathf.Clamp(Time.timeScale * 2f, 0, 4);
            Sound.AdjustPitch();
        }
        if (Input.GetKeyDown(KeyCode.KeypadMinus)) {
            Time.timeScale = Mathf.Clamp(Time.timeScale * 0.5f, 0.0625f, 4);
            Sound.AdjustPitch();
        }        
        if (Input.GetKeyDown("l")) {
            score += 1000;
            CheckIfButtonsAreEnabled();
        }
        if (Input.GetKeyDown("w") || Input.GetKeyDown(KeyCode.UpArrow)) {
            up = true;
            playerCharacter.Jump();
        }
        if (Input.GetKeyUp("w") || Input.GetKeyUp(KeyCode.UpArrow)) {
            up = false;
        }
        if (Input.GetKeyDown("a") || Input.GetKeyDown(KeyCode.LeftArrow)) {
            left = true;
        }
        if (Input.GetKeyUp("a") || Input.GetKeyUp(KeyCode.LeftArrow)) {
            left = false;
        }
        if (Input.GetKeyDown("s") || Input.GetKeyDown(KeyCode.DownArrow)) {
            down = true;
        }
        if (Input.GetKeyUp("s") || Input.GetKeyUp(KeyCode.DownArrow)) {
            down = false;
        }
        if (Input.GetKeyDown("d") || Input.GetKeyDown(KeyCode.RightArrow)) {
            right = true;
        }
        if (Input.GetKeyUp("d") || Input.GetKeyUp(KeyCode.RightArrow)) {
            right = false;
        }
        // ***********************************  MOUSE CLICKING ******************
        //
        // ** This is my nasty raycasting code, beware, it confuses me too
        // ** First it raycasts at the menu layer
        // ** Layers are assigned by the layerdropdown on the top right of any game object
        // ** If the raycast is true then the mouse is over the menu and so it doesn't think about clicking on the battlefield
        // ** though if "dragging" is true it knows that 
        //
        Vector3 mousePos = Input.mousePosition; // ** this is the mouse position in Screen coordinates
        if (dragging) {
            InteractWithBattlefield(mousePos);
        } else if (!InteractWithMenu(mousePos)){
            InteractWithBattlefield(mousePos);
        }
        
        if (overButton == null && disabledButton == null) {
            tooltipText.text = "";
        }
    }

    bool InteractWithMenu(Vector3 mousePos) {
        RaycastHit hit;
        Ray ray = mainCamera.ScreenPointToRay(mousePos);
        if (!dragging && Physics.Raycast(ray, out hit, 200, menuLayer)) {
            //
            //** if the mouse is over the menu **
            //
            Button currentButton = hit.collider.gameObject.GetComponent(typeof(Button)) as Button; 
            // ** because we know the mouse is over the menu there will be a button class on the collider the raycast hits
            if (overButton != null) {
                if (currentButton != overButton) {
                    overButton.Out(); // ** the mouse is over a different button
                }
            }
            if (currentButton.isEnabled) {// ** the button the mouse is over is enabled
                disabledButton = null;
                if (currentButton != overButton) {
                    DisplayTooltipText(currentButton.type, true); // ** display the tooltip
                }
                overButton = currentButton;
                overButton.Over();
                if (Input.GetButtonDown("Fire1")) { // ** the mouse is pressed down, the button is enabled
                    if (downButton != null) {
                        downButton.OtherDown();
                    }
                    downButton = overButton;
                    downButton.Down();
                }
                if (Input.GetButtonUp("Fire1")) {// ** the mouse is clicked, the button is enabled
                    //
                    // ** perform the action related to the button, this is defined by the Button.type variable
                    //
                    if (downButton == currentButton) {
                        PerformAction(currentButton.type);
                    }
                    AllButtonsOut();
                }
            } else {
                // ** the button the mouse is over is disabled 
                disabledButton = currentButton;
                DisplayTooltipText(currentButton.type, false);
            }
            return true;
        } else {
            return false;
        }
    }
    void InteractWithBattlefield(Vector3 mousePos) {

        // ****  MOUSE IS NOT OVER THE MENU  ****
        disabledButton = null;
        if (overButton != null) {
            overButton.Out();
            overButton = null;
        }
        // ** The Camera.ScreenToWorldPoint function is incredibly useful... it takes in Screen coordinates with the z being the distance from the camera
        Vector3 newWorldPoint = mainCamera.ScreenToWorldPoint(Input.mousePosition + Vector3.forward * 100);
        //
        if (Input.GetButtonDown("Fire1")) {

            // ** I decided to do this with an overlap sphere because it hits a larger area than the raycast
            //
            Collider[] shipColliders = Physics.OverlapSphere(newWorldPoint, 3, shipsLayer); // ** Only checks on the ships layer
            if (shipColliders.Length > 0) {
                // ** If the overlap sphere overlapped colliders
                for (int i = 0; i < shipColliders.Length; i++) {
                   // selectedShip = shipColliders[i].gameObject.GetComponent(typeof(Ship)) as Ship;
                    selectedShip = shipColliders[i].gameObject;
                    if (selectedShip != null) {
                      //  if (!selectedShip.evil) {
                            // ** if it finds a non evil ship underneath the mouse
                            break;
                      //  }
                    }
                }
                if (selectedShip != null) {
                   // if (selectedShip.evil) {
                   //     selectedShip = null;
                  //  } else {
                        // ** start dragging, a ship has been clicked on
                        // ** now the line will drag around following the mouse
                        dragging = true;
                        draggingLine.enabled = true;
                   // }
                }
            }
        }
        //** though I went back to a ray for the mouse up event, You can actually make ships follow each other, but I don't think users will do that so it doesn't matter if it is tricky
        //
        RaycastHit hit;
        if (Input.GetButtonUp("Fire1") && selectedShip != null) {
            // ** It only performs the button click procedure if there is a selected ship
            dragging = false;
            draggingLine.enabled = false;
            Ray ray = mainCamera.ScreenPointToRay(mousePos);
            GameObject targetShip = null;
            if (Physics.Raycast(ray, out hit, 2000, shipsLayer)) { // ** checks to see if a ship is clicked upon              
               // targetShip = hit.collider.gameObject.GetComponent(typeof(Ship)) as Ship;
                targetShip = hit.collider.gameObject;
            }
            if (targetShip != null) {
                if (targetShip != selectedShip) {
                    selectedShip.SendMessage("SetDestination", targetShip.transform);
                  //  selectedShip.SetDestination(targetShip.transform);
                } else {
                    selectedShip.SendMessage("SetDestination", newWorldPoint);
                 //   selectedShip.SetDestination(newWorldPoint);
                }
            } else {
                // ** if no ship is clicked upon then it tells the selected ship just to go to the position underneath the mouse
                selectedShip.SendMessage("SetDestination", newWorldPoint);
                //selectedShip.SetDestination(newWorldPoint);
            }
            Map.WarpSparks(newWorldPoint, 7); // ** Sparkles YAY!
            Sound.PlaySound(SoundType.Beep, 0.3f);
            selectedShip = null;
            AllButtonsOut();
        }
        // ** it makes the dragging line go away if the ship dies
        // ** I think this occasionally doesn't work properly, there may be scenarios I'm not aware of
        if (dragging) {
            if (selectedShip != null) {
                draggingLine.SetPosition(0, selectedShip.transform.position);
                draggingLine.SetPosition(1, newWorldPoint);
            } else {
                // ** make line disappear 
                draggingLine.enabled = false;
                dragging = false;
            }
        }

    }

    void Pause() {
        if (Time.timeScale > 0.1f) {
            Time.timeScale = 0.1f;
        } else {
            Time.timeScale = 1f;
        }
        Sound.AdjustPitch();
        Sound.PlaySound(SoundType.Beep, 0.2f);
    }
    static void CheckIfButtonsAreEnabled() {
        // ** It enables/disables buttons according to score
        // ** it could be more elegant, but it works
        //
        for (int i = 0; i < buttons.Length; i++) {
            if (buttons[i].type == ButtonType.AwesomeShip) {
                if (score < 1500) {
                    if (downButton == buttons[i]) {
                        downButton.OtherDown();
                    }
                    if (overButton == buttons[i]) {
                        overButton.Out();
                    }
                    buttons[i].Enable(false);
                   
                } else {
                    buttons[i].Enable(true);
                }

            } else if (buttons[i].type == ButtonType.BasicShip) {
                if (score < 100) {
                    if (downButton == buttons[i]) {
                        downButton.OtherDown();
                    }
                    if (overButton == buttons[i]) {
                        overButton.Out();
                    }
                    buttons[i].Enable(false);
                   
                } else {
                    buttons[i].Enable(true);
                }
            } else if (buttons[i].type == ButtonType.MissileShip) {
                if (score < 500) {
                    if (downButton == buttons[i]) {
                        downButton.OtherDown();
                    }
                    if (overButton == buttons[i]) {
                        overButton.Out();
                    }
                    buttons[i].Enable(false);

                } else {
                    buttons[i].Enable(true);
                }
            } else if (buttons[i].type == ButtonType.GunShip) {
                if (score < 300) {
                    if (downButton == buttons[i]) {
                        downButton.OtherDown();
                    }
                    if (overButton == buttons[i]) {
                        overButton.Out();
                    }
                    buttons[i].Enable(false);

                } else {
                    buttons[i].Enable(true);
                }
            } 
        }
    }
    void AllButtonsOut() {
        // ** sometimes all the buttons must be unselected
        if (downButton != null) {
            downButton.OtherDown();
        }
        if (overButton != null) {
            overButton.Out();
        }
        disabledButton = null;
        overButton = null;
        downButton = null;
        disabledButton = null;
    }
    void DisplayTooltipText(ButtonType type, bool enabled) {
        //** this changes the tooltipMaterial text and colour
        if (enabled) {
            tooltipMaterial.SetColor("_Color", new Color(0.8f, 1, 0));
        } else {
            tooltipMaterial.SetColor("_Color", new Color(0.8f, 0, 0));
        }
        if (type == ButtonType.AwesomeShip) {            
            tooltipText.text = "Warp In Battleship (1500)";
        } else if (type == ButtonType.BasicShip) {
            tooltipText.text = "Warp In Fighter (100)";
        } else if (type == ButtonType.GunShip) {
            tooltipText.text = "Warp In Gunship (300)";
        } else if (type == ButtonType.MissileShip) {
            tooltipText.text = "Warp In Bomber (500)";
        } 
    }

    void PerformAction(ButtonType type) {
        // ** when a Button class is clicked on this function is called
        // ** this is different to the Unity GUI buttons, there is more effort involved, but it may be a little faster, though maybe not (because of the raycasting)
        // ** but this way the buttons could be any object in the game, they could be attached to the ships (like for context menus)
        // ** though they're not attached to ships, they may as well probably be GuiButtons
        //
        if (type == ButtonType.AwesomeShip) {
            Map.SpawnShip(ShipType.AlliedBattleShip, - Vector3.up *15);
            score -= 1500;
            Sound.PlaySound(SoundType.Sexy, 0.5f);
        } else if (type == ButtonType.MissileShip) {
            Map.SpawnShip(ShipType.AlliedMissileCarrier, - Vector3.up * 15);
            score -= 500;
            Sound.PlaySound(SoundType.Sexy, 0.5f);
        } else if (type == ButtonType.GunShip) {
            Map.SpawnShip(ShipType.AlliedGunShip, - Vector3.up * 15);
            score -= 300;
            Sound.PlaySound(SoundType.Sexy, 0.5f);
        } else if (type == ButtonType.BasicShip) {
            Map.SpawnShip(ShipType.AlliedFighter,- Vector3.up * 15);
            score -= 100;
            Sound.PlaySound(SoundType.Sexy, 0.5f);
        } else if (type == ButtonType.Exit) {
            Application.LoadLevel(0);
        }
        CheckIfButtonsAreEnabled();
    }
}
